EXPORTS
	cln_CreateClient
	cln_DestroyClient
	cln_SelectLayer
	cln_ReleaseLayer
	cln_BroadCloseLayers
	cln_SelectRegion
	cln_SelectMask
	cln_UnSelectMask
	cln_GetDictionary
	cln_GetAttributesFormat
	cln_GetNextObject
	cln_GetRasterInfo
	cln_GetObject
	cln_GetObjectIdFromCoord
	cln_UpdateDictionary
	cln_GetGlobalBound
	cln_SetServerLanguage
	cln_GetServerProjection
	cln_SetTclProc
	cln_GetTclProc
	cln_GetClientIdFromURL
	cln_GetURLList
	cln_SetRegionCaches
	cln_LoadCache
	cln_ReleaseCache
	cln_SetCompression
	ecs_DistanceObject
	ecs_DistanceObjectWithTolerance
	ecs_SetTolerance
	ecs_DistanceMBR
	ecs_DistanceSegment
	ecs_SetError
	ecs_SetSuccess
	ecs_AdjustResult
	ecs_SetGeoRegion
	ecs_SetText
	ecs_AddText
	ecs_SetRasterInfo
	ecs_AddRasterInfoCategory
	ecs_SetObjAttributeFormat
	ecs_AddAttributeFormat
	ecs_SetGeomPoint
	ecs_SetGeomText
	ecs_SetGeomLine
	ecs_SetGeomArea
	ecs_SetGeomAreaRing
	ecs_SetGeomMatrix
	ecs_SetGeomMatrixWithArray
	ecs_SetGeomImage
	ecs_SetGeomImageWithArray
	ecs_SetObjectId
	ecs_SetObjectAttr
	ecs_CleanUp
	ecs_CleanUpObject
	ecs_ResultInit
	ecs_CalcObjectMBR
	ecs_freeSplitURL
	ecs_SplitURL
	ecs_Backslash
	ecs_FindElement
	ecs_CopyAndCollapse
	ecs_SplitList
	svr_CreateServer
	svr_DestroyServer
	svr_SelectLayer
	svr_ReleaseLayer
	svr_SelectRegion
	svr_GetDictionary
	svr_GetAttributesFormat
	svr_GetNextObject
	svr_GetRasterInfo
	svr_GetObject
	svr_GetObjectIdFromCoord
	svr_UpdateDictionary
	svr_GetServerProjection
	svr_GetGlobalBound
	svr_SetServerLanguage
	svr_SetRasterConversion
	svr_CloseLayer
	svr_BroadCloseLayers
	ecs_SetLayer
	ecs_GetLayer
	ecs_FreeLayer
	ecs_RemoveDir
	ecs_GetDefaultInfo
	xdr_ecs_Family
	xdr_ecs_Region
	xdr_ecs_Coordinate
	xdr_ecs_FeatureRing
	xdr_ecs_Area
	xdr_ecs_Line
	xdr_ecs_Point
	xdr_ecs_Matrix
	xdr_ecs_Image
	xdr_ecs_Text
	xdr_ecs_Node
	xdr_ecs_Edge
	xdr_ecs_TopoLevel
	xdr_ecs_Face
	xdr_ecs_AreaPrim
	xdr_ecs_Geometry
	xdr_ecs_Object
	xdr_ecs_AttributeFormat
	xdr_ecs_ObjAttribute
	xdr_ecs_ObjAttributeFormat
	xdr_ecs_Category
	xdr_ecs_RasterInfo
	xdr_ecs_ResultType
	xdr_ecs_ResultUnion
	xdr_ecs_Result
	xdr_ecs_LayerSelection
	xdr_ecs_RasterConversion
	xdr_ecs_Compression
	xdr_ecs_ProxyCreateServer
	ecs_begin_ellipsoid_polygon_area
	ecs_planimetric_polygon_area
	ecs_ellipsoid_polygon_area
	ecs_geodesic_distance
	ecs_CalculateCentroid
	ecs_CopyObject
	ecs_CopyResult
	ecs_FreeObject
	ecs_TileInitialize
	ecs_TileGetLine
	ecs_TileClearBuffer
	cln_BlockOGDI
	cln_UnBlockOGDI
	cln_GetVersion
	cln_CheckExtension
	cln_GetLayerCapabilities
	cln_LoadCapabilities
	ecs_GetRGBFromPixel
	ecs_GetPixelFromRGB
	ecs_SetReportErrorFunction
	ecs_ShouldStopOnError
	ecs_SetErrorShouldStop
    ecs_TileDeleteAllLines
