
#ifndef H_MUSE
#define H_MUSE

#ifndef INCL_XVTH
#include "xvt.h"
#endif

#ifndef H_MUSE1
#include "muse1.h"
#endif
#ifndef H_SYSTEM_D
#include "system_d.h"
#endif
#ifndef H_COLOR_DEF
#include "color_d.h"
#endif
#ifndef H_DTCC_D
#include "dtcc_d.h"
#endif
#ifndef H_PHIGS_D
#include "phigs_d.h"
#endif
#ifndef H_UNIT_DEF
#include "unit_d.h"
#endif
#ifndef H_MAPDOC_D
#include "mapdoc_d.h"
#endif
#ifndef H_SUNR_DEF
#include "sunras_d.h"
#endif
#ifndef H_SQLLIB_D
#include "sqllib_d.h"
#endif
#ifndef H_CLI_SR_D
#include "cli_sr_d.h"
#endif
#ifndef H_COORD_DEF
#include "coord_d.h"
#endif
#ifndef H_ELLIPS_DEF
#include "ellips_d.h"
#endif
#ifndef H_DATUM_DEF
#include "datum_d.h"
#endif
#ifndef H_COORD_FUNC
#include "coord_f.h"
#endif
#ifndef H_ELLIPS_FUNC
#include "ellips_f.h"
#endif
#ifndef H_DATUM_F
#include "datum_f.h"
#endif
#ifndef H_MUSE_IPC
#include "muse_ipc.h"
#endif
#ifndef H_MUSE_SQL
#include "muse_sql.h"
#endif
#ifndef H_SYSTEM_F
#include "system_f.h"
#endif
#ifndef H_COLOR_F
#include "color_f.h"
#endif
#ifndef H_DTCC_F
#include "dtcc_f.h"
#endif
#ifndef H_DTCC_FN
#include "dtcc_fn.h"            /* DT&CC functions pulled from old MUSEDEF.H */
#endif
#ifndef H_PHIGS_F
#include "phigs_f.h"
#endif
#ifndef H_RASTER_F
#include "raster_f.h"
#endif
#ifndef H_SUNR_FUNC
#include "sunras_f.h"
#endif
#ifndef H_UNIT_FUNC
#include "unit_f.h"
#endif
#ifndef H_VIEW_FUNC
#include "view_f.h"
#endif
#ifndef H_MAPDOC_F
#include "mapdoc_f.h"
#endif
#ifndef H_VEC_D
#include "vec_d.h"
#endif
#ifndef H_VEC_F
#include "vec_f.h"
#endif
#ifndef H_VECTOR
#include "vector.h"
#endif
#ifndef XVTCM_INCL
#include "xvtcm.h"
#endif
#ifndef H_SQLLIB_F
#include "sqllib_f.h"
#endif
#ifndef H_CLI_SR_F
#include "cli_sr_f.h"
#endif

#endif                          /* H_MUSE */
