#ifndef H_CLI_SR_D
#define H_CLI_SR_D

#define AUTO_COORD   10
#define GUI_COORD    11

#define AUTO_DATUM   20
#define GUI_DATUM    21

#define AUTO_ELLIPS  30
#define GUI_ELLIPS   31

#define GET_POINT    40

#define GET_CHILD    88

#define TERMINATE    99

#endif /* CLI_SR_D */
