/*
 * Please do not edit this file.
 * It was generated using rpcgen.
 */

#include <memory.h> /* for memset */
#include "ecs.h"

/* Default timeout can be changed using clnt_control() */
static struct timeval TIMEOUT = { 25, 0 };

ecs_Result *
createserver_1(char **argp, CLIENT *clnt)
{
	static ecs_Result clnt_res;

	memset((char *)&clnt_res, 0, sizeof (clnt_res));
	if (clnt_call(clnt, CREATESERVER,
		(xdrproc_t) xdr_wrapstring, (caddr_t) argp,
		(xdrproc_t) xdr_ecs_Result, (caddr_t) &clnt_res,
		TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&clnt_res);
}

ecs_Result *
destroyserver_1(void *argp, CLIENT *clnt)
{
	static ecs_Result clnt_res;

	memset((char *)&clnt_res, 0, sizeof (clnt_res));
	if (clnt_call(clnt, DESTROYSERVER,
		(xdrproc_t) xdr_void, (caddr_t) argp,
		(xdrproc_t) xdr_ecs_Result, (caddr_t) &clnt_res,
		TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&clnt_res);
}

ecs_Result *
selectlayer_1(ecs_LayerSelection *argp, CLIENT *clnt)
{
	static ecs_Result clnt_res;

	memset((char *)&clnt_res, 0, sizeof (clnt_res));
	if (clnt_call(clnt, SELECTLAYER,
		(xdrproc_t) xdr_ecs_LayerSelection, (caddr_t) argp,
		(xdrproc_t) xdr_ecs_Result, (caddr_t) &clnt_res,
		TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&clnt_res);
}

ecs_Result *
releaselayer_1(ecs_LayerSelection *argp, CLIENT *clnt)
{
	static ecs_Result clnt_res;

	memset((char *)&clnt_res, 0, sizeof (clnt_res));
	if (clnt_call(clnt, RELEASELAYER,
		(xdrproc_t) xdr_ecs_LayerSelection, (caddr_t) argp,
		(xdrproc_t) xdr_ecs_Result, (caddr_t) &clnt_res,
		TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&clnt_res);
}

ecs_Result *
selectregion_1(ecs_Region *argp, CLIENT *clnt)
{
	static ecs_Result clnt_res;

	memset((char *)&clnt_res, 0, sizeof (clnt_res));
	if (clnt_call(clnt, SELECTREGION,
		(xdrproc_t) xdr_ecs_Region, (caddr_t) argp,
		(xdrproc_t) xdr_ecs_Result, (caddr_t) &clnt_res,
		TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&clnt_res);
}

ecs_Result *
getdictionnary_1(void *argp, CLIENT *clnt)
{
	static ecs_Result clnt_res;

	memset((char *)&clnt_res, 0, sizeof (clnt_res));
	if (clnt_call(clnt, GETDICTIONNARY,
		(xdrproc_t) xdr_void, (caddr_t) argp,
		(xdrproc_t) xdr_ecs_Result, (caddr_t) &clnt_res,
		TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&clnt_res);
}

ecs_Result *
getattributeformat_1(void *argp, CLIENT *clnt)
{
	static ecs_Result clnt_res;

	memset((char *)&clnt_res, 0, sizeof (clnt_res));
	if (clnt_call(clnt, GETATTRIBUTEFORMAT,
		(xdrproc_t) xdr_void, (caddr_t) argp,
		(xdrproc_t) xdr_ecs_Result, (caddr_t) &clnt_res,
		TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&clnt_res);
}

ecs_Result *
getnextobject_1(void *argp, CLIENT *clnt)
{
	static ecs_Result clnt_res;

	memset((char *)&clnt_res, 0, sizeof (clnt_res));
	if (clnt_call(clnt, GETNEXTOBJECT,
		(xdrproc_t) xdr_void, (caddr_t) argp,
		(xdrproc_t) xdr_ecs_Result, (caddr_t) &clnt_res,
		TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&clnt_res);
}

ecs_Result *
getrasterinfo_1(void *argp, CLIENT *clnt)
{
	static ecs_Result clnt_res;

	memset((char *)&clnt_res, 0, sizeof (clnt_res));
	if (clnt_call(clnt, GETRASTERINFO,
		(xdrproc_t) xdr_void, (caddr_t) argp,
		(xdrproc_t) xdr_ecs_Result, (caddr_t) &clnt_res,
		TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&clnt_res);
}

ecs_Result *
getobject_1(char **argp, CLIENT *clnt)
{
	static ecs_Result clnt_res;

	memset((char *)&clnt_res, 0, sizeof (clnt_res));
	if (clnt_call(clnt, GETOBJECT,
		(xdrproc_t) xdr_wrapstring, (caddr_t) argp,
		(xdrproc_t) xdr_ecs_Result, (caddr_t) &clnt_res,
		TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&clnt_res);
}

ecs_Result *
getobjectidfromcoord_1(ecs_Coordinate *argp, CLIENT *clnt)
{
	static ecs_Result clnt_res;

	memset((char *)&clnt_res, 0, sizeof (clnt_res));
	if (clnt_call(clnt, GETOBJECTIDFROMCOORD,
		(xdrproc_t) xdr_ecs_Coordinate, (caddr_t) argp,
		(xdrproc_t) xdr_ecs_Result, (caddr_t) &clnt_res,
		TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&clnt_res);
}

ecs_Result *
updatedictionary_1(char **argp, CLIENT *clnt)
{
	static ecs_Result clnt_res;

	memset((char *)&clnt_res, 0, sizeof (clnt_res));
	if (clnt_call(clnt, UPDATEDICTIONARY,
		(xdrproc_t) xdr_wrapstring, (caddr_t) argp,
		(xdrproc_t) xdr_ecs_Result, (caddr_t) &clnt_res,
		TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&clnt_res);
}

ecs_Result *
getserverprojection_1(void *argp, CLIENT *clnt)
{
	static ecs_Result clnt_res;

	memset((char *)&clnt_res, 0, sizeof (clnt_res));
	if (clnt_call(clnt, GETSERVERPROJECTION,
		(xdrproc_t) xdr_void, (caddr_t) argp,
		(xdrproc_t) xdr_ecs_Result, (caddr_t) &clnt_res,
		TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&clnt_res);
}

ecs_Result *
getglobalbound_1(void *argp, CLIENT *clnt)
{
	static ecs_Result clnt_res;

	memset((char *)&clnt_res, 0, sizeof (clnt_res));
	if (clnt_call(clnt, GETGLOBALBOUND,
		(xdrproc_t) xdr_void, (caddr_t) argp,
		(xdrproc_t) xdr_ecs_Result, (caddr_t) &clnt_res,
		TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&clnt_res);
}

ecs_Result *
setserverlanguage_1(u_int *argp, CLIENT *clnt)
{
	static ecs_Result clnt_res;

	memset((char *)&clnt_res, 0, sizeof (clnt_res));
	if (clnt_call(clnt, SETSERVERLANGUAGE,
		(xdrproc_t) xdr_u_int, (caddr_t) argp,
		(xdrproc_t) xdr_ecs_Result, (caddr_t) &clnt_res,
		TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&clnt_res);
}

ecs_Result *
setrasterconversion_1(ecs_RasterConversion *argp, CLIENT *clnt)
{
	static ecs_Result clnt_res;

	memset((char *)&clnt_res, 0, sizeof (clnt_res));
	if (clnt_call(clnt, SETRASTERCONVERSION,
		(xdrproc_t) xdr_ecs_RasterConversion, (caddr_t) argp,
		(xdrproc_t) xdr_ecs_Result, (caddr_t) &clnt_res,
		TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&clnt_res);
}

ecs_Result *
createproxyserver_1(ecs_ProxyCreateServer *argp, CLIENT *clnt)
{
	static ecs_Result clnt_res;

	memset((char *)&clnt_res, 0, sizeof (clnt_res));
	if (clnt_call(clnt, CREATEPROXYSERVER,
		(xdrproc_t) xdr_ecs_ProxyCreateServer, (caddr_t) argp,
		(xdrproc_t) xdr_ecs_Result, (caddr_t) &clnt_res,
		TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&clnt_res);
}

ecs_Result *
setcompression_1(ecs_Compression *argp, CLIENT *clnt)
{
	static ecs_Result clnt_res;

	memset((char *)&clnt_res, 0, sizeof (clnt_res));
	if (clnt_call(clnt, SETCOMPRESSION,
		(xdrproc_t) xdr_ecs_Compression, (caddr_t) argp,
		(xdrproc_t) xdr_ecs_Result, (caddr_t) &clnt_res,
		TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&clnt_res);
}
